
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "conkywx.h"
#include <time.h>

struct tm tmv2, tmv3;

char* months[] = {
  "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
} ;


int get_rst (int object, double date, double ourlong, double ourlat,
			int year, int month, int day,
			double *Rise, double *Set, double *Transit)
//get rise, set and transit times of object sun or moon
{
	double sl, lod1;          //sin lat
	double cl;          //cos lat
	double xe, ye, z1, z2;
	double riseTime = 0, setTime = 0, transitTime = 0;
	double above = 0;
	double flagriseset = 0;
	double ym = 0, y0, yp;
	double sinho[2];        //0 IS MOON, 1 IS SUN
	int doesRise = 0, doesSet = 0, doesTransit = 0, hour = 1, check = 0;
	int nz = 0;
	char eventName[22];
	char objectms[10];
//******************************************************************************
// init some values
	sl = sind(ourlat); cl = cosd(ourlat);  //sin and cos of latitude
	sinho[0] = .002327;         //moonrise - average diameter used
	sinho[1] = -0.014544;       //sunrise - classic value for refraction

	ym = sinalt(object, date, hour - 1, ourlong, cl, sl) - sinho[object];

	if (ym > 0) {
		above = 1;
	}
	else {
		above = 0;
	}
//start rise-set loop
	do
	{
		y0 = sinalt(object, date, hour, ourlong, cl, sl) - sinho[object];
		yp = sinalt(object, date, hour + 1, ourlong, cl, sl) - sinho[object];
		xe = 0;
		ye = 0;
		z1 = 0;
		z2 = 0;
		nz = 0;
		quad (ym, y0, yp, &xe, &ye, &z1, &z2, &nz);

		switch (nz)
		{
			case 0:	//nothing  - go to next time slot
				break;
			case 1:                      //simple rise / set event
				if (ym < 0) {       //must be a rising event
					riseTime = hour + z1;
					doesRise = 1;
				}
				else {	//must be setting
					setTime = hour + z1;
					doesSet = 1;
				}
				break;
			case 2:                      //rises and sets within interval
				if (ye < 0) {       //minimum - so set then rise
					riseTime = hour + z2;
					setTime = hour + z1;
				}
				else {    //maximum - so rise then set
					riseTime = hour + z1;
					setTime = hour + z2;
				}
				doesRise = 1;
				doesSet = 1;
				break;
				}

		ym = yp;     //reuse the ordinate in the next interval
		hour = hour + 2;
		check = (doesRise * doesSet);
	}
	while ((hour != 25) && (check != 1));
	// end rise-set loop
//GET TRANSIT TIME
	hour = 0; //reset hour
	//transitTime = get_transit(object, date, hour, ourlong, ourlat);

	if (object == 0){
		snprintf(objectms, 8, "moon");
		//char objectms[] = "moon";
	}
	else {
		snprintf(objectms, 8, "sun");
		//char objectms[] = "sun";
	}
	if (doesRise == 1) {
		*Rise = riseTime;
		snprintf(eventName, 6, "rise");
		display_event_time(riseTime, objectms, eventName, year, month, day);
		flagriseset = 1;
	}
	else {
		if (above == 1) {
			printf ("\n%s_rise=\"A.A.H.\"", objectms );
			printf ("\n%s_rise_T=\"NA\"", objectms );
			// printf ("=\"A.A.H.\"");
			flagriseset = 1;
		}
		else {
			if ( flagriseset == 1 ) {
				printf ("\n%s_rise=\"A.A.H.\"", objectms );
				printf ("\n%s_rise_T=\"NA\"", objectms );
				// printf ("=\"A.A.H.\"");
				//flagriseset = 0;
			} else {
				printf ("\n%s_rise=\"A.B.H.\"", objectms );
				printf ("\n%s_rise_T=\"NA\"", objectms );
				// printf ("=\"A.B.H.\"");
			}
		}
	}
	if (doesSet == 1) {
		*Set = setTime;
		snprintf(eventName, 6, "set");
		display_event_time(setTime, objectms, eventName, year, month, day);
	}
	else {
		if (above == 1) {
			if ( flagriseset == 1 ) {
				printf ("\n%s_set=\"A.A.H.\"", objectms );
				printf ("\n%s_set_T=\"NA\"", objectms );
				// printf ("=\"A.A.H.\"");
				//flagriseset = 0;
			} else {
				printf ("\n%s_set=\"A.B.H.\"", objectms );
				printf ("\n%s_set_T=\"NA\"", objectms );
				// printf ("=\"A.B.H.\"");
			}
		} else {
			if ( flagriseset == 1 ) {
				printf ("\n%s_set=\"A.A.H.\"", objectms );
				printf ("\n%s_set_T=\"NA\"", objectms );
				printf ("=\"A.A.H.\"");
				//flagriseset = 0;
			} else {
				printf ("\n%s_set=\"A.B.H.\"", objectms );
				printf ("\n%s_set_T=\"NA\"", objectms );
				// printf ("=\"A.B.H.\"");
			}
		}

	}
	if ( flagriseset == 1 ) {
		transitTime = get_transit(object, date, hour, ourlong, ourlat);
		if ((transitTime != 0.0) && (transitTime < 24)) {
			doesTransit = 1;
			flagriseset = 0;
		}
	}
	if (doesTransit == 1) {
		*Transit = transitTime;
		snprintf(eventName, 20, "mer_trans");
		display_event_time(transitTime, objectms, eventName, year, month, day);
	}
	else {
		*Transit = 0.0;
		printf ("\n%s_mer_trans", objectms );
		printf ("=\"N.M.T.\"");
	}

	// length of day
	lod1 = setTime - riseTime ;
	if (object == 1) {
		if ( above == 1) {
			if ( riseTime > setTime ) { lod1 = 24 + lod1; }
			if ((lod1 > 24) || (lod1 == 0)) {
				printf( "\nlength_day=\"%2.2dh %2.2dm\"", 24, 00  );
				printf( "\nlength_day_T=\"%1d.%2.2d\"", 24, 00  );
			} else {
				printf( "\nlength_day=\"%2.2dh %2.2dm\"", HOURS(lod1), MINUTES(lod1) );
				printf( "\nlength_day_T=\"%1d.%2.2d\"", HOURS(lod1), MINUTES(lod1) );
			}
		} else {
			printf( "\nlength_day=\"%2.2dh %2.2dm\"", HOURS(lod1), MINUTES(lod1) );
			printf( "\nlength_day_T=\"%1d.%2.2d\"", HOURS(lod1), MINUTES(lod1) );
		}
		// printf( "\nlength_day_raw=\"%d\"", (int)((lod1-floor(lod1))*1000000) );
		printf( "\nlength_day_raw=\"%1.6f\"", lod1 );
	}
return 0;
}

int print_situation ( int sit, char local,
					char* up_string, double up,
					char* down_string, double down,
					char* always_up, char* always_down,
					int year, int month, int day )
{
	// prints the twilight zone data
	// struct tm tmv3;

	switch( sit )
	{
	case 0:
		if (!local)
			// printf( "%s\"%2.2d%2.2d %sUTC\"\n%s\"%2.2d%2.2d %sUTC\"\n",
			printf( "\n%s\"%2.2d%2.2d%s\"\n%s\"%2.2d%2.2d%s\"",
			up_string,
			TMOD(HOURS(up)), MINUTES(up), DAYSOFF(up),
			down_string,
			TMOD(HOURS(down)), MINUTES(down), DAYSOFF(down) );
			up = TMOD(up+timezone_offset/3600);
			down = TMOD(down+timezone_offset/3600);

		if (local)
			tmv2 = mk_tm(year, month, day, HOURS(up), MINUTES(up), 00);
			strftime (buffer, SIZE, tformat, &tmv2);
			tmv3 = mk_tm(year, month, day, HOURS(down), MINUTES(down), 00);
			strftime (buffer2, SIZE, tformat, &tmv3);
			printf( "\n%s\"%s\"\n%s\"%s\"",
			up_string,
			// HOURS(up), MINUTES(up),
			buffer,
			down_string,
			// HOURS(down), MINUTES(down) );
			buffer2 );
		break;
	case +1:
		printf( "\n%s", always_up );
		break;
	case -1:
		printf( "\n%s", always_down );
		break;
	}
	return sit;
}

int print_everything(int year, int month, int day,
					double ourlat, double ourlong,
					double tz, char local)
	{
	// double daylen, civlen, lod, lovl; // , nautlen, astrlen;
	double lovl; // , nautlen, astrlen;
	double rise, set, civ_start, civ_end, naut_start, naut_end,
	astr_start, astr_end;
	int    rs, civ, naut, astr;

	ourlong = 0.0 - ourlong;  //equations use west longitude negative convention

	civ  = civil_twilight ( year, month, day, ourlong, ourlat, &civ_start, &civ_end );
	naut = nautical_twilight ( year, month, day, ourlong, ourlat, &naut_start, &naut_end );
	astr = astronomical_twilight ( year, month, day, ourlong, ourlat, &astr_start, &astr_end );

	lovl = civ_end - civ_start ;
	printf( "\nlength_visible_light=\"%2.2dh %2.2dm\"", HOURS(lovl), MINUTES(lovl)  );
	printf( "\nlength_visible_light_T=\"%1d.%2.2d\"", HOURS(lovl), MINUTES(lovl)  );

	print_situation ( civ, local,
			"civ_twilt_R=", civ_start+tz,
			"civ_twilt_S=", civ_end+tz,
			"civ_twilt_R=\"No Sun Set\"",
			"civ_twilt_R=\"No Sun Rise\"",
			year, month, day );
	print_situation ( naut, local,
			"nau_twilt_R=", naut_start+tz,
			"nau_twilt_S=", naut_end+tz,
			"nau_twilt_R=\"No Sun Set\"",
			"nau_twilt_R=\"No Sun Rise\"",
			year, month, day );

	print_situation ( astr, local,
			"ast_twilt_R=", astr_start+tz,
			"ast_twilt_S=", astr_end+tz,
			"ast_twilt_R=\"No Sun Set\"",
			"ast_twilt_R=\"No Sun Rise\"",
			year, month, day );

	return 0;

}
/******************************************************************************/
double get_julian_date(int year, int month, int day, double UT)
{
	double locJD, a, b, c, d, e, f, g;
	a = modf((month + 9)/12, &b);
	a = modf((7 * (year + b))/4, &c);
	a = modf((275 * month)/9, &d);
	e = 367 * year - c + d + day + 1721013.5 + UT/24;
	f = (100 * year + month - 190002.5);
	g = f/fabs(f);
	locJD = e - 0.5 * g + 0.5;
	// printf( "\nJUlian-date=\"%9.9f\"\n", locJD );
return (locJD);
}

/******************************************************************************/
// prints rise set and transits
void display_event_time (double time, char objname[6], char event [6], int year, int month, int day)
{
	tmv2 = mk_tm(year, month, day, HOURS(time), MINUTES(time), 00);
	strftime (buffer, SIZE, tformat, &tmv2);
	printf("\n%s_%s=\"%s\"",
		objname, event,
		buffer );
	// printf("\n%s_%s_T=\"%2.2d%2.2d\"",
	printf("\n%s_%s_T=\"%1d.%2.2d\"",
		objname, event,
		HOURS(time),
		MINUTES(time) );

return;
}

/******************************************************************************/
float ipart (double x)
//returns the true integer part, even for negative numbers
{
	float a;
	if (x != 0) {
	a = x/fabs(x) * floor(fabs(x));
	}
	else {
		a=0;
	}
return a;
}

/******************************************************************************/
double fpart (double x)
//returns fractional part of a number
{
	x = x - floor(x);
	if ( x < 0) {
		x = x + 1;
	}
return x;
}

/******************************************************************************/
double sinalt (int object, double mjd0, double hour, double ourlong, double cphi,
			double sphi )
/*
returns sine of the altitude of either the sun or the moon given the modified
julian day number at midnight UT and the hour of the UT day, the longitude of
the observer, and the sine and cosine of the latitude of the observer
*/
{
	double loc_sinalt;   //sine of the altitude, return value;
	double ra = 0.0;
	double dec = 0.0;
	double instant, t;
	double lha;			//hour angle
	instant = mjd0 + hour / 24.0;
	t = (instant - 51544.5) / 36525;
	if (object == 0) {
		get_moon_pos (t, &ra, &dec);
	}
	else {
		get_sun_pos (t, &ra, &dec);
	}
	lha = 15.0 * (lmst(instant, ourlong) - ra);    //hour angle of object
	loc_sinalt = sphi * sind(dec) + cphi * cosd(dec) * cosd(lha);
return (loc_sinalt);
}

/******************************************************************************/
double lmst (double mjd, double ourlong)
//returns the local siderial time for the modified julian date and longitude
{
	double value;
	float mjd0;
	double ut;
	double t;
	double gmst;
	mjd0  = ipart(mjd);
	ut    = (mjd - mjd0) * 24;
	t     = (mjd0 - 51544.5) / 36525;
	gmst  = 6.697374558 + 1.0027379093 * ut;
	gmst  = gmst + (8640184.812866 + (.093104 - .0000062 * t) * t) * t / 3600;
	value = 24.0 * fpart((gmst - ourlong / 15.0) / 24.0);
return (value);
}

/******************************************************************************/
double azalt (int object, double mjd0, double hour, double ourlat, double ourlong )
/*
returns sine of the altitude of either the sun or the moon given the modified
julian day number at midnight UT and the hour of the UT day, the longitude of
the observer, and the sine and cosine of the latitude of the observer
*/
{
	double loc_sinalt, ALT;   //sine of the altitude, return value;
	double ra = 0.0;
	double dec = 0.0;
	double instant, t, mydec;
	double lha, AZ, t1;

	instant = mjd0 + hour;
	t = (instant - 51544.5) / 36525;
	if (object == 0) {
		get_moon_pos (t, &ra, &dec);
	}
	else {
		get_sun_pos (t, &ra, &dec);
	}
	lha        = 15 * (zalmst(instant, ourlong) - ra);    //hour angle of object
	lha        = ( lha < 0 ) ? 360 + lha : lha ;
	loc_sinalt = sind(ourlat) * sind(dec) + cosd(ourlat) * cosd(dec) * cosd(lha);
	ALT        = asind(loc_sinalt);
	AZ         = (dsin(dec) - (dsin(ALT) * dsin(ourlat))) / (dcos(ALT) * dcos(ourlat)) ;
	AZ         = (acosd(AZ));
	AZ         = ( dsin(lha) < 0 ) ? AZ : 360 - AZ ;

	if (object == 0) {
		printf( "\nmoon_az=\"%3.9f\"", AZ );
		// printf( "\nmoon_ra=\"%3.9f\"", ra );
		// printf( "\nmoon_ra=\"%3.9f\"", ra*15 );
		printf( "\nmoon_dec=\"%3.9f\"", dec );
		// printf( "\nmoon_HA=\"%3.9f\"", lha/15 );
		if ( ourlat > dec ) {
			mydec = 1;
		}
		else {
			mydec = 0;
		}
		printf( "\nmoon_dec_state=\"%1.0f\"", mydec );
	}
	else {
		printf( "\nsun_az=\"%3.9f\"", AZ );
		// printf( "\nsun_ra=\"%3.9f\"", ra );
		// printf( "\nsun_ra=\"%3.9f\"", ra*15 );
		printf( "\nsun_dec=\"%3.9f\"", dec );
		// t1 = zalmst(instant, ourlong);
		// printf( "\nsun_lmst=\"%3.9f\"", t1 );
		// printf( "\nsun_lmst=\"%3.9f\"", t1*15 );
	}

return 0;
}

/******************************************************************************/
double zalmst (double mjd, double ourlong)
//returns the local siderial time for the modified julian date and longitude
{
	double value;
	float mjd0;
	double ut;
	double t;
	double gmst;
	mjd0  = ipart(mjd);
	ut    = (mjd - mjd0) * 24;
	t     = (mjd - 51544.5) / 36525;
	gmst  = 6.697374558 + 1.0027379093 * ut;
	gmst  = gmst + (8640184.812866 + (.093104 - .0000062 * t) * t) * t / 3600;
	value = 24.0 * fpart((gmst) / 24.0);
return (value);
}

/******************************************************************************/
void get_sun_pos (double t, double *ra, double *dec)
/*
Returns RA and DEC of Sun to roughly 1 arcmin for few hundred years either side
of J2000.0
*/
{
	double COSEPS = 0.91748;
	double SINEPS = 0.39778;
	double m, dL, L, rho, sl;
	double RA, DEC;
	double x, y, z;
	m   = twoPI * fpart(0.993133 + 99.997361 * t);        //Mean anomaly
	dL  = 6893 * sin(m) + 72 * sin(2 * m);          //Eq centre
	L   = twoPI * fpart(0.7859453 + m / twoPI + (6191.2 * t + dL) / 1296000);
	sl  = sin(L);
	x   = cos(L);
	y   = COSEPS * sl;
	z   = SINEPS * sl;
	rho = sqrt(1 - z * z);
	DEC = (360 / twoPI) * atan2(z , rho);
	RA  = (48 / twoPI) * atan2(y , (x + rho));
	if (RA < 0) {
		RA = RA + 24;
	}
	*ra  = RA;
	*dec = DEC;
return;
}

/******************************************************************************/
void get_moon_pos (double t, double *ra, double *dec)
/*
returns ra and dec of Moon to 5 arc min (ra) and 1 arc min (dec) for a few
centuries either side of J2000.0 Predicts rise and set times to within minutes
for about 500 years in past - TDT and UT time difference may become significant
for long times
*/
{
	double ARC    = 206264.8062;
	double COSEPS = 0.91748;
	double SINEPS = 0.39778;
	double L0, L, LS, d, F;
	L0    = fpart(.606433 + 1336.855225 * t);         //'mean long Moon in revs
	L     = twoPI * fpart(.374897 + 1325.55241 * t);  //'mean anomaly of Moon
	LS    = twoPI * fpart(.993133 + 99.997361 * t);   //'mean anomaly of Sun
	d     = twoPI * fpart(.827361 + 1236.853086 * t); //'diff longitude sun and moon
	F     = twoPI * fpart(.259086 + 1342.227825 * t); //'mean arg latitude
	//' longitude correction terms
	double dL, h;
	dL    = 22640 * sin(L) - 4586 * sin(L - 2 * d);
	dL    = dL + 2370 * sin(2 * d) + 769 * sin(2 * L);
	dL    = dL - 668 * sin(LS) - 412 * sin(2 * F);
	dL    = dL - 212 * sin(2 * L - 2 * d) - 206 * sin(L + LS - 2 * d);
	dL    = dL + 192 * sin(L + 2 * d) - 165 * sin(LS - 2 * d);
	dL    = dL - 125 * sin(d) - 110 * sin(L + LS);
	dL    = dL + 148 * sin(L - LS) - 55 * sin(2 * F - 2 * d);
	//' latitude arguments
	double S, N, lmoon, bmoon;
	S     = F + (dL + 412 * sin(2 * F) + 541 * sin(LS)) / ARC;
	h     = F - 2 * d;
	//' latitude correction terms
	N     = -526 * sin(h) + 44 * sin(L + h) - 31 * sin(h - L) - 23 * sin(LS + h);
	N     = N + 11 * sin(h - LS) - 25 * sin(F - 2 * L) + 21 * sin(F - L);
	lmoon = twoPI * fpart(L0 + dL / 1296000); //  'Lat in rads
	bmoon = (18520 * sin(S) + N) / ARC;       //  'long in rads
	//' convert to equatorial coords using a fixed ecliptic
	double CB, x, V, W, y, Z, rho, DEC, RA;
	CB    = cos(bmoon);
	x     = CB * cos(lmoon);
	V     = CB * sin(lmoon);
	W     = sin(bmoon);
	y     = COSEPS * V - SINEPS * W;
	Z     = SINEPS * V + COSEPS * W;
	rho   = sqrt(1.0 - Z * Z);
	DEC   = (360.0 / twoPI) * atan2(Z , rho);
	RA    = (48.0 / twoPI) * atan2(y , (x + rho));
	if (RA < 0) {
		RA = RA + 24.0;
	}
	*ra = RA;
	*dec = DEC;
return;
}

/******************************************************************************/
void quad (double ym, double y0, double yp, double *xe, double *ye, double *z1,
			double *z2, int *nz)
/*
finds a parabola through three points and returns values of coordinates of
extreme value (xe, ye) and zeros if any (z1, z2) assumes that the x values are
-1, 0, +1
*/
{
	double a, b, c, dx, dis, XE, YE, Z1, Z2;
	int NZ;
	NZ = 0;
	XE = 0;
	YE = 0;
	Z1 = 0;
	Z2 = 0;
	a = .5 * (ym + yp) - y0;
	b = .5 * (yp - ym);
	c = y0;
	XE = (0.0 - b) / (a * 2.0); //              'x coord of symmetry line
	YE = (a * XE + b) * XE + c; //      'extreme value for y in interval
	dis = b * b - 4.0 * a * c;   //    'discriminant
	//more nested if's
	if ( dis > 0.000000 ) {                 //'there are zeros
		dx = (0.5 * sqrt(dis)) / (fabs(a));
		Z1 = XE - dx;
		Z2 = XE + dx;
		if (fabs(Z1) <= 1) {
			NZ = NZ + 1 ;   // 'This zero is in interval
		}
		if (fabs(Z2) <= 1) {
			NZ = NZ + 1  ;   //'This zero is in interval
		}
		if (Z1 < -1) {
			Z1 = Z2;
		}
	}
	*xe = XE;
	*ye = YE;
	*z1 = Z1;
	*z2 = Z2;
	*nz = NZ;
return;
}

/******************************************************************************/
double get_transit (int object, double mjd0, int hour, double ourlong, double ourlat)
{
	double ra = 0.0;
	double dec = 0.0;   //not used here but the pos functions return it.
	double instant, t;
	double lha;			//local hour angle
	double loc_transit;	// transit time, return value.
	int min = 0;
	int hourarray[24];
	int minarray[60];
	double LA;  //local angle
	int sLA;    //sign of angle
	double mintime;

//loop through all 24 hours of the day and store the sign of the angle in an array
//actually loop through 25 hours if we reach the 25th hour with out a transit then no transit condition today.

	// object 0 is moon

	while (hour < 25.0)
	{
		instant = mjd0 + hour / 24.0;
		t = (instant - 51544.5) / 36525;
		if (object == 0) {
			get_moon_pos (t, &ra, &dec);

		}
		else {
			get_sun_pos (t, &ra, &dec);

		}
		lha = (lmst(instant, ourlong) - ra);

		LA = lha * 15.04107;    //convert hour angle to degrees
		sLA = LA/fabs(LA);      //sign of angle

		hourarray[hour] = LA;
		//hourarray[hour] = (int)(floor(LA));

 //test 1
 //if (object == 1) {
	 //printf( "\nRA Dec=\"%d %d %f %f\"", hour, hourarray[hour], LA, lha );
 //}


		hour++;
	}
//search array for the point when the angle first goes from negative to positive
	int i = 0;
	while (i < 25)
	{
		loc_transit = i;
		// if (hourarray[i] - hourarray[i+1] == -2) {
		if (((hourarray[i] >= -14 ) && (hourarray[i] <= 0)) || (hourarray[i] > 345)) {
		// if (hourarray[i] + hourarray[i+1] > 0) {
			//we found our hour
			break;
		}
 //test 2
 //if (object == 1) {
	 //printf( "\nhourarray=\"%d %d %d\"", i, hourarray[i], loc_transit);
 //}


		i++;
	}
	//check for no transit, return zero
	if (loc_transit > 23) {
		// no transit today
		loc_transit = 0.0;
		return loc_transit;
	}

//loop through all 60 minutes of the hour and store sign of the angle in an array
	mintime = loc_transit;
	while (min < 60)
	{
		instant = mjd0 + mintime / 24.0;
		t = (instant - 51544.5) / 36525;
		if (object == 0) {
			get_moon_pos (t, &ra, &dec);
		}
		else {
			get_sun_pos (t, &ra, &dec);
		}
		lha = (lmst(instant, ourlong) - ra);
		LA = lha * 15.04107;
		sLA = (int)(LA/fabs(LA));
		minarray[min] = sLA;

// test 3
 //if (object == 1) {
	 //printf( "\nRA Dec=\"%d %d %f %f\"", min, minarray[min], LA, lha );
 //}

		min++;
		mintime = mintime + 0.016667;		//increment 1 minute
		// mintime = mintime + 0.000833333;		//increment 1/2 minute
		// mintime = mintime + 0.00002777777778;		//increment 1/2 minute
	}

	i = 0;
	while (i < 60)
	{
		if (minarray[i] - minarray[i+1] == -2) {
		//we found our min
		break;
		}
		i++;
		loc_transit = loc_transit + 0.016667;

// test 4
  //if (object == 0) {
	 //printf( "\nhourarray=\"%d %d\"", i,  loc_transit);
   //}



	}
return (loc_transit);
}


/******************************************************************************/

int __sunriset__( int year, int month, int day, double ourlong, double ourlat,
				  double altit, int upper_limb, double *trise, double *tset )
//***************************************************************************/
/* Note: year,month,date = calendar date, 1801-2099 only.             */
/*       Eastern longitude positive, Western longitude negative       */
/*       Northern latitude positive, Southern latitude negative       */
/*       The longitude value IS critical in this function!            */
/*       altit = the altitude which the Sun should cross              */
/*               Set to -35/60 degrees for rise/set, -6 degrees       */
/*               for civil, -12 degrees for nautical and -18          */
/*               degrees for astronomical twilight.                   */
/*         upper_limb: non-zero -> upper limb, zero -> center         */
/*               Set to non-zero (e.g. 1) when computing rise/set     */
/*               times, and to zero when computing start/end of       */
/*               twilight.                                            */
/*        *rise = where to store the rise time                        */
/*        *set  = where to store the set  time                        */
/*                Both times are relative to the specified altitude,  */
/*                and thus this function can be used to comupte       */
/*                various twilight times, as well as rise/set times   */
/* Return value:  0 = sun rises/sets this day, times stored at        */
/*                    *trise and *tset.                               */
/*               +1 = sun above the specified "horizon" 24 hours.     */
/*                    *trise set to time when the sun is at south,    */
/*                    minus 12 hours while *tset is set to the south  */
/*                    time plus 12 hours. "Day" length = 24 hours     */
/*               -1 = sun is below the specified "horizon" 24 hours   */
/*                    "Day" length = 0 hours, *trise and *tset are    */
/*                    both set to the time when the sun is at south.  */
/*                                                                    */
/**********************************************************************/
{
	double  d,  /* Days since 2000 Jan 0.0 (negative before) */
	sr,         /* Solar distance, astronomical units */
	sRA,        /* Sun's Right Ascension */
	sdec,       /* Sun's declination */
	sradius,    /* Sun's apparent radius */
	t,          /* Diurnal arc */
	tsouth,     /* Time when Sun is at south */
	sidtime;    /* Local sidereal time */

	int rc = 0; /* Return cde from function - usually 0 */

	/* Compute d of 12h local mean solar time */
	d = days_since_2000_Jan_0(year,month,day) + 0.5 - ourlong/360.0;

	/* Compute local sideral time of this moment */
	sidtime = revolution( GMST0(d) + 180.0 + ourlong );

	/* Compute Sun's RA + Decl at this moment */
	sun_RA_dec( d, &sRA, &sdec, &sr );

	/* Compute time when Sun is at south - in hours UT */
	tsouth = 12.0 - rev180(sidtime - sRA)/15.0;

	/* Compute the Sun's apparent radius, degrees */
	sradius = 0.2666 / sr;

	/* Do correction to upper limb, if necessary */
	if ( upper_limb )
		altit -= sradius;

	/* Compute the diurnal arc that the Sun traverses to reach */
	/* the specified altitide altit: */
	{
		double cost;
		cost = ( sind(altit) - sind(ourlat) * sind(sdec) ) /
			( cosd(ourlat) * cosd(sdec) );
		if ( cost >= 1.0 )
			rc = -1, t = 0.0;       /* Sun always below altit */
		else if ( cost <= -1.0 )
			rc = +1, t = 12.0;      /* Sun always above altit */
		else
			t = acosd(cost)/15.0;   /* The diurnal arc, hours */
	}

	/* Store rise and set times - in hours UT */
	*trise = tsouth - t;
	*tset  = tsouth + t;

	return rc;
}  /* __sunriset__ */

/* This function computes the Sun's position at any instant */

void sunpos( double d, double *lon, double *r )
/******************************************************/
/* Computes the Sun's ecliptic longitude and distance */
/* at an instant given in d, number of days since     */
/* 2000 Jan 0.0.  The Sun's ecliptic latitude is not  */
/* computed, since it's always very near 0.           */
/******************************************************/
{
	  double M,         /* Mean anomaly of the Sun */
			 w,         /* Mean longitude of perihelion */
						/* Note: Sun's mean longitude = M + w */
			 e,         /* Eccentricity of Earth's orbit */
			 E,         /* Eccentric anomaly */
			 x, y,      /* x, y coordinates in orbit */
			 v;         /* True anomaly */

	  /* Compute mean elements */
	  M = revolution( 356.0470 + 0.9856002585 * d );
	  w = 282.9404 + 4.70935E-5 * d;
	  e = 0.016709 - 1.151E-9 * d;

	  /* Compute true longitude and radius vector */
	  E = M + e * RADEG * sind(M) * ( 1.0 + e * cosd(M) );
	  x = cosd(E) - e;
	  y = sqrt( 1.0 - e*e ) * sind(E);
	  *r = sqrt( x*x + y*y );              /* Solar distance */
	  v = atan2d( y, x );                  /* True anomaly */
	  *lon = v + w;                        /* True solar longitude */
	  if ( *lon >= 360.0 )
			*lon -= 360.0;                   /* Make it 0..360 degrees */
}

void sun_RA_dec( double d, double *RA, double *dec, double *r )
{
  double lon, obl_ecl;
  double xs, ys;
  double xe, ye, ze;

  /* Compute Sun's ecliptical coordinates */
  sunpos( d, &lon, r );

  /* Compute ecliptic rectangular coordinates */
  xs = *r * cosd(lon);
  ys = *r * sind(lon);
  //zs = 0; /* because the Sun is always in the ecliptic plane! */

  /* Compute obliquity of ecliptic (inclination of Earth's axis) */
  obl_ecl = 23.4393 - 3.563E-7 * d;

  /* Convert to equatorial rectangular coordinates - x is unchanged */
  xe = xs;
  ye = ys * cosd(obl_ecl);
  ze = ys * sind(obl_ecl);

  /* Convert to spherical coordinates */
  *RA = atan2d( ye, xe );
  *dec = atan2d( ze, sqrt(xe*xe + ye*ye) );

}  /* sun_RA_dec */



/******************************************************************/
/* This function reduces any angle to within the first revolution */
/* by subtracting or adding even multiples of 360.0 until the     */
/* result is >= 0.0 and < 360.0                                   */
/******************************************************************/

#define INV360    ( 1.0 / 360.0 )

double revolution( double x )
/*****************************************/
/* Reduce angle to within 0..360 degrees */
/*****************************************/
{
	  return( x - 360.0 * floor( x * INV360 ) );
}  /* revolution */

double rev180( double x )
/*********************************************/
/* Reduce angle to within -180..+180 degrees */
/*********************************************/
{
	  return( x - 360.0 * floor( x * INV360 + 0.5 ) );
}  /* revolution */


/*******************************************************************/
/* This function computes GMST0, the Greenwhich Mean Sidereal Time */
/* at 0h UT (i.e. the sidereal time at the Greenwhich meridian at  */
/* 0h UT).  GMST is then the sidereal time at Greenwich at any     */
/* time of the day.  I've generelized GMST0 as well, and define it */
/* as:  GMST0 = GMST - UT  --  this allows GMST0 to be computed at */
/* other times than 0h UT as well.  While this sounds somewhat     */
/* contradictory, it is very practical:  instead of computing      */
/* GMST like:                                                      */
/*                                                                 */
/*  GMST = (GMST0) + UT * (366.2422/365.2422)                      */
/*                                                                 */
/* where (GMST0) is the GMST last time UT was 0 hours, one simply  */
/* computes:                                                       */
/*                                                                 */
/*  GMST = GMST0 + UT                                              */
/*                                                                 */
/* where GMST0 is the GMST "at 0h UT" but at the current moment!   */
/* Defined in this way, GMST0 will increase with about 4 min a     */
/* day.  It also happens that GMST0 (in degrees, 1 hr = 15 degr)   */
/* is equal to the Sun's mean longitude plus/minus 180 degrees!    */
/* (if we neglect aberration, which amounts to 20 seconds of arc   */
/* or 1.33 seconds of time)                                        */
/*                                                                 */
/*******************************************************************/

double GMST0( double d )
{
	  double sidtim0;
	  /* Sidtime at 0h UT = L (Sun's mean longitude) + 180.0 degr  */
	  /* L = M + w, as defined in sunpos().  Since I'm too lazy to */
	  /* add these numbers, I'll let the C compiler do it for me.  */
	  /* Any decent C compiler will add the constants at compile   */
	  /* time, imposing no runtime or code overhead.               */
	  sidtim0 = revolution( ( 180.0 + 356.0470 + 282.9404 ) +
						  ( ( 0.9856002585 + 4.70935E-5   ) * d) );
	  return sidtim0;
}  /* GMST0 */

